﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ThexWinStream
{
    public partial class Form1 : Form
    {
        ushort[,] palleteArr = new ushort[1001, 3];
        Bitmap paletteImg = new Bitmap("Iron.png");
        bool liveStream = true;

        public Form1()
        {
            InitializeComponent();
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            IntPtr hWnd = this.Handle;

            int bRes = DLLHelper.UsbOpenDevice(hWnd, 0);
            if (bRes == 1)
            {
                textBox1.Text = "Connected!";
            }
            else
            {
                textBox1.Text = "Err:" + bRes.ToString();
            }

            bRes = DLLHelper.ReadFlashData(0);
            if (bRes == 1)
            {
                textBox1.Text = "Flash read completed";
                btnStream.Enabled = true;
                btnDisconnect.Enabled = true;
                btnCalibrate.Enabled = true;
            }
            else
            {
                textBox1.Text = "Err:" + bRes.ToString();
            }
        }

        private void btnStream_Click(object sender, EventArgs e)
        {
            liveStream = true;
            StartStreaming streamData = new StartStreaming(StreamData);
            streamData.BeginInvoke(null, null);
        }

        delegate void StartStreaming();

        void StreamData()
        {
            ushort[] imageData = new ushort[288 * 384];
            byte[] bArr = new byte[288 * 384];
            Bitmap b = new Bitmap(384, 288, PixelFormat.Format8bppIndexed);
            ColorPalette ncp = b.Palette;
            Rectangle BoundsRect = new Rectangle(0, 0, 384, 288);
            BitmapData bmpData;
            IntPtr ptr;

            for (int i = 0; i < 256; i++)
                ncp.Entries[i] = Color.FromArgb(255, i, i, i);
            b.Palette = ncp;

            while (liveStream)
            {
                DLLHelper.RecvImage(imageData, 0);

                for (int n = 0; n < 288 * 384; n++)
                {
                    bArr[n] = (byte)(imageData[n] >> 8);
                }
                bmpData = b.LockBits(BoundsRect, ImageLockMode.WriteOnly, PixelFormat.Format8bppIndexed);
                ptr = bmpData.Scan0;

                Marshal.Copy(bArr, 0, ptr, bmpData.Stride * 288);
                b.UnlockBits(bmpData);

                RefreshImage(b);
            }
        }

        delegate void RefreshImageDelegate(Bitmap bmpData);

        void RefreshImage(Bitmap bmpData)
        {
            if (pictureBox1.InvokeRequired == false)
            {
                pictureBox1.Image = bmpData;
                if (btnStop.Enabled == false) btnStop.Enabled = true;
                if (btnCalibrate.Enabled == true && liveStream == true) btnCalibrate.Enabled = false;
            }
            else
            {
                RefreshImageDelegate showProgress = new RefreshImageDelegate(RefreshImage);
                BeginInvoke(showProgress, new object[] { bmpData });
            }
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            liveStream = false;
            btnCalibrate.Enabled = true;
        }

        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            DLLHelper.UsbCloseDevice(0);
            textBox1.Text = "Disconected!";
        }

        private void btnCalibrate_Click(object sender, EventArgs e)
        {
            bool bRes = DLLHelper.ShutterCalibrationOn(0);
            if (bRes)
            {
                textBox1.Text = "Calibration completed";
            }
            else
            {
                textBox1.Text = "Err:" + bRes.ToString();
            }
        }
    }
}
